#!/usr/bin/env python3

###############################################################################
# telegram_api
# Copyright © 2020 Valentin Basel <valentinbasel@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################

import json 
import requests


class TELEGRAM(object):

    """Docstring for TELEGRAM. """

    def __init__(self, token):
        """TODO: to be defined. """ 
        self._TOKEN = token
        self._URL = "https://api.telegram.org/bot{}/".format(self._TOKEN)
        self.__tipos_msg = ["forward_from",
                            "forward_from_chat",
                            "forward_from_message_id",
                            "forward_signature",
                            "forward_sender_name",
                            "reply_to_message",
                            "edit_date",
                            "media_group_id",
                            "author_signature", 
                            "text", 	
                            "entities",
                            "caption_entities",
                            "audio",
                            "document",
                            "animation",
                            "game",
                            "photo",
                            "sticker",
                            "video",
                            "voice",
                            "video_note",
                            "caption",
                            "contact",
                            "location",
                            "venue",
                            "poll",
                            "new_chat_members",
                            "left_chat_member",
                            "new_chat_title",
                            "new_chat_photo",
                            "delete_chat_photo",
                            "group_chat_created",
                            "supergroup_chat_created",
                            "channel_chat_created",
                            "migrate_to_chat_id",
                            "migrate_from_chat_id",
                            "pinned_message",
                            "invoice",
                            "successful_payment",
                            "connected_website",
                            "passport_data",
                            "reply_markup"]
        # el numero total de mensajes
        self._num_act = self.__ultimo_msg()

    def hay_mensajes(self):
        """TODO: Docstring for hay_mensajes.
        :returns: TODO

        """
        num = self.__ultimo_msg()
        if num > self._num_act:
            self._num_act = num
            return True
        else:
            return False

    def __ultimo_msg(self):
        """TODO: Docstring for ultimo_msg.
        :returns: TODO

        """
        act = self.actualizar()
        num_act = len(act["result"])
        return num_act

    def __obtener_url(self, url):
        response = requests.get(url)
        content = response.content.decode("utf8")
        return content

    def __obtener_json_desde_url(self, url):
        content = self.__obtener_url(url)
        js = json.loads(content)
        return js

    def actualizar(self):
        url = self._URL + "getUpdates"
        js = self.__obtener_json_desde_url(url)
        return js

    def tipo_msg(self):
        act = self.actualizar()
        ultima_act = self._num_act - 1
        text = act["result"][ultima_act]["message"]
        tipo_msg = list(text.keys())
        return tipo_msg[4]

    def obtener_ultimo_mensaje(self):
        act = self.actualizar()
        ultima_act = self._num_act - 1
        text = act["result"][ultima_act]["message"]
        tipo_msg = list(text.keys())
        text = act["result"][ultima_act]["message"][tipo_msg[4]]

        chat_id = act["result"][ultima_act]["message"]["chat"]["id"]
        return (chat_id, text)

    def enviar_mensaje(self, chat_id, texto):
        url = self._URL + \
              "sendMessage?text={}&chat_id={}".format(texto, chat_id)
        self.__obtener_url(url)
        return True

    def enviar_imagen(self, chat_id, arch):
        """TODO: Docstring for enviar_imagen.
        :returns: TODO

        """
        url = self._URL + "sendPhoto"
        try:
            requests.post(url,
                          data={"chat_id": chat_id},
                          files={'photo': open(arch, 'rb')}
                          ) 
        except Exception as e:
            # raise e
            print(e)
            return False
        return True




#!/usr/bin/env python3
# -*- coding: utf-8 -*-


###############################################################################
# Clase TELEGRAMBOT como capa de abstracción para domoticaro
# Copyright © 2020 Valentin Basel <valentinbasel@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
from .telegram_api import TELEGRAM
from .emulador import EMULADOR


class BOT(object):

    """La clase BOT proporciona una capa de abstracción que permite tener
        varios formatos de comunicación (telegram, irc, off-line)"""

    def __init__(self, tipo, arg=None):
        if tipo == "telegram":
            self.__chatbot = TELEGRAM(arg)
        elif tipo == "emulador":
            self.__chatbot = EMULADOR(arg)
        else:
            print(f'error, el valor {tipo} no es un tipo reconocido')
            exit(1)

    def actualizar(self):
        flag = self.__chatbot.actualizar()
        return flag

    def enviar_texto(self,  chat_id, texto):
        flag = self.__chatbot.enviar_mensaje(chat_id, texto)
        return flag

    def enviar_imagen(self, chat_id, arch):
        """TODO: Docstring for enviar_imagen.
        :returns: TODO

        """
        flag = self.__chatbot.enviar_imagen(chat_id, arch)
        return flag

    def hay_mensaje(self):
        flag = self.__chatbot.hay_mensajes()
        return flag

    def recibir_ultimo_mensaje(self):
        chat_id, texto = self.__chatbot.obtener_ultimo_mensaje()
        return (chat_id, texto)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
#
# interface de abstracción para el hardware basado en micro controladores 
# 18F4550 con bootloader pinguino V4.0
#
# Copyright © 2019 Valentín Basel <valentinbasel@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################
from .base import BASE_HARDWARE
from .relay import RELE
from .analogico import ANALOGICO
from .lcd import LCD


class PIC18F4550(BASE_HARDWARE):

    """Docstring for 18F4550.
    La clase PIC18F4550 es una capa de abstracción diseñada para poder usar
    de forma transparente la comunicación entre python y la placa ICARO
    basada en el micro controlador 18f4550.
    hereda de la clase BASE_HARDWARE los metodos para la comunicación serie CDC

    los metodos heredados de BASE_HARDWARE:

    self.iniciar()
    self.cerrar()
    """
    def __init__(self, puerto):
        print("inicio 18f4550")
        BASE_HARDWARE.__init__(self, puerto)
        self.rele1 = RELE(1, self._enviar)
        self.rele2 = RELE(2, self._enviar)
        self.rele3 = RELE(3, self._enviar)
        self.rele4 = RELE(4, self._enviar)
        self.analogico_1 = ANALOGICO(1, self._enviar)
        self.lcd = LCD(self._enviar)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
###############################################################################
#
# clase para implementación de ELIZA
# Copyright © 2019 Valentín Basel <valentinbasel@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################
import re
import random

class ELIZA(object):
    """ Class doc """
    

    # reflejos sobre el sujeto
    reflections = {
        "estoy": "estas",
        "soy": "eres",
        "fui": "fuiste",
        "yo": "tu",
        "deberia": "deberias",
        "tengo": "tendrias",
        "quiero": "quieres",
        "mio": "tuyo",
        "eres": "soy",
        "tu tienes": "yo tengo",
        "tu seras": "yo sere",
        "tuyo": "mio",
        "vuestro": "nuestro",
        "sobre ti": "sobre mi",
        "sobre mi": "sobre ti",
        "tu":"mi",
        "vos":"yo",

    }

    # patrones psicologicos más probables
    psychobabble = [
        [r'necesito(.*)',
         ["¿Por qué necesitas {0}?",
          "¿Realmente te ayudaría a obtener {0}?",
          "¿Está seguro de que necesitas {0}?"]],

        [r'por que no puedes ([^\?]*)\??' or r'por qué no puedes ([^\?]*)\??',
         ["¿De verdad crees que no lo hago? {0}?",
          "Tal vez eventualmente lo haré. {0}.",
          "¿De verdad quieres que yo {0}?"]],

        [r'¿Por que no puedo ([^\?]*)\??' or r'por que no puedo ([^\?]*)\??' or r'¿Por qué no puedo ([^\?]*)\??' or r'Por qué no puedo ([^\?]*)\??',
         ["¿Crees que deberías ser capaz de {0}?",
          "Si pudieras {0}, ¿qué harías?",
          "No sé - ¿por qué no puedes {0}?",
          "¿Lo has intentado realmente?",
          "Si no eres capaz de {0} entonces eso te define cómo un autentico marica!"]],

        [r'no puedo(.*)',
         ["¿Cómo sabes que no puedes{0}?",
          "Tal vez podría {0} si lo intentó.",
          "¿Qué se necesita para que usted {0}?"]],

        [r'yo soy (.*)',
         ["¿Has venido a mí porque eres  {0}?",
          "¿Cuánto tiempo has estado {0}?",
          "¿Cómo te sientes acerca de ser {0}?"]],

        [r'estoy (.*)',
         ["¿Cómo te hace sentir{0}?",
          "¿Te gusta ser {0}?",
          "¿Por qué me dices que eres{0}?",
          "¿Por qué crees que eres  {0}?"]],

        [r'¿ ([^\?]*)\??',
         ["¿Por qué importa si soy {0}?",
          "¿Lo preferirías si no fuera {0}?",
          "Tal vez usted piensa que soy.",
          "Puedo ser {0} - ¿qué piensas?"]],

        [r'¿qué (.*)' or r'qué (.*)' or r'que (.*)',
         ["¿Por qué preguntas?",
          "¿Cómo le ayudaría una respuesta a eso?",
          "¿Qué piensas?"]],

        [r'¿cómo (.*)' or r'cómo (.*)' or r'como (.*)',
         ["¿Cómo se supone?",
          "Tal vez puedas responder a tu propia pregunta.",
          "¿Qué estás realmente pidiendo?"]],

        [r'porque (.*)',
         ["¿Es esa la verdadera razón?",
          "¿Qué otras razones vienen a la mente?",
          "¿Esa razón se aplica a cualquier otra cosa?",
          "Si {0}, ¿qué otra cosa debe ser verdadera?"]],

        [r'(.*) lo siento (.*)',
         ["Hay muchas veces cuando no se necesita disculpa.",
          "¿Qué sentimientos tiene cuando se disculpa?"]],

        [r'hola(.*)',
         ["Hola ... me alegro de que puedas pasar por hoy.",
          "¿Hola, cómo estas hoy?",
          "Hola, ¿cómo te sientes hoy?"]],

        [r'pienso (.*)',
         ["¿Dudas {0}?",
          "¿De verdad piensas eso?",
          "Pero no estás seguro {0}?"]],

        [r'(.*) amigo (.*)',
         ["Cuéntame más sobre tus amigos.",
          "Cuando piensas en un amigo, ¿qué te viene a la mente?",
          "¿Por qué no me hablas de un amigo desde la infancia?"]],

        [r'si',
         ["Pareces muy seguro.",
          "Está bien, pero ¿puedes hacer algo?"]],

        [r'(.*) ordenador(.*)',
         ["¿De verdad estás hablando de mí?",
          "¿Parece extraño hablar con una computadora?",
          "¿Cómo te hacen sentir las computadoras?",
          "¿Se siente amenazado por las computadoras?"]],

        [r'eso es (.*)',
         ["¿Crees que es {0}?",
          "Tal vez sea {0} - ¿qué piensas?",
          "Si fuera {0}, ¿qué harías?",
          "Podría ser que {0}."]],

        [r'es eso (.*)',
         ["Pareces muy seguro.",
          "Si te dijera que probablemente no es {0}, ¿qué sentirías?"]],

        [r'¿puedes ([^\?]*)\??',
         ["¿Qué te hace pensar que no puedo {0}?",
          "Si pudiera {0}, ¿entonces qué?",
          "¿Por qué me preguntas si puedo (0)?"]],

        [r'puedo ([^\?]*)\??',
         ["No puedes querer {0}.",
          "¿Quieres ser capaz de {0}?",
          "Si pudieras {0}, ¿cierto?"]],

        [r'tu eres (.*)' or r'¿tu eres (.*)',
         ["¿Por qué crees que soy {0}?",
          "¿Te gusta pensar que soy {0}?",
          "Tal vez usted quiere que sea.",
          "¿Tal vez estás hablando de ti?"]],

        [r'¿eres ture (.*)',
         ["¿Por qué dices que soy {0}?",
          "¿Por qué crees que soy {0}?",
          "¿Estamos hablando de ti o de mí?"]],

        [r'¿no lo hago (.*)',
         ["No realmente {0}?",
          "¿Por qué no {0}?",
          "¿Quieres {0}?"]],

        [r'me siento (.*)',
         ["Bueno, cuéntame más sobre estos sentimientos.",
          "¿Sientes a menudo {0}?",
          "¿Cuándo sientes normalmente {0}?",
          "Cuando sientes {0}, ¿qué haces?"]],

        [r'yo tengo (.*)',
         ["¿Por qué me dices que tienes {0}?",
          "¿Realmente tienes {0}?",
          "Ahora que tienes {0}, ¿qué harás después?"]],

        [r'yo debo (.*)',
         ["¿Podría explicar por qué lo haría?",
          "¿Por que lo harias?",
          "¿Quién más sabe que lo harías?"]],

        [r'esta ahí (.*)',
         ["¿Crees que hay {0}?",
          "Es probable que haya {0}.",
          "¿Te gustaría tener {0}?"]],

        [r'mi (.*)',
         ["Ya veo, tu {0}.",
          "¿Por qué dices que tu {0}?",
          "Cuando tu {0}, ¿cómo te sientes?"]],

        [r'tu (.*)',
         ["debemos estar discutiendo contigo, no conmigo.",
          "¿Por qué dices eso de mí?",
          "¿Por qué te importa si yo {0}?"]],

        [r'por qué (.*)',
         ["¿Por qué no me dices la razón por la cual {0}?",
          "¿Por qué crees que {0}?"]],

        [r'quiero que (.*)',
         ["¿Qué significaría para ti si tienes {0}?",
          "¿Por qué quieres {0}?",
          "¿Qué harías si tuvieras {0}?",
          "Si tienes {0}, entonces ¿qué harías?"]],

        [r'(.*) madre(.*)',
         ["Cuéntame más sobre tu madre.",
          "¿Cómo era tu relación con tu madre?",
          "¿Cómo te sientes con tu madre?",
          "¿Cómo se relaciona esto con tus sentimientos hoy?",
          "Las buenas relaciones familiares son importantes."]],

        [r'(.*) padre(.*)',
         ["Cuéntame más sobre tu padre.",
          "¿Cómo te hizo sentir tu padre?",
          "¿Cómo te sientes con tu padre?",
          "¿Su relación con su padre se relaciona con sus sentimientos hoy?",
          "¿Tiene problemas para mostrar afecto con su familia?"]],

        [r'(.*) niño(.*)',
         ["¿Tenías amigos íntimos cuando era niño?",
          "¿Cuál es tu recuerdo favorito de la niñez?",
          "¿Recuerdas algún sueño o pesadilla desde la infancia?",
          "¿Alguna vez te han molestado los otros niños?",
          "¿Cómo crees que tus experiencias infantiles se relacionan con tus sentimientos hoy?"]],

        [r'(.*)\?',
         ["¿Porque preguntas eso?",
          "Por favor, considere si puede responder a su propia pregunta.",
          "¿Quizás la respuesta está dentro de ti?",
          "¿Por qué no me lo dices?"]],

        [r'adios',
         ["Gracias por hablar conmigo.",
          "Adiós.",
          "Gracias, eso será $ 150. ¡Que tengas un buen día!"]],

        [r'(.*)',
         ["Por favor, cuéntame más.",
          "Vamos a cambiar de enfoque un poco ... Háblame de tu familia.",
          "¿Puedes profundizar sobre eso?",
          "¿Por qué dices eso {0}?",
          "Ya veo.",
          "Muy interesante.",
          "{0}.",
          "Ya veo, ¿y qué te dice eso?",
          "¿Cómo te hace sentir eso?",
          "¿Cómo te sientes cuando dices eso?"]]
    ]

    def __init__ (self):
        """ Class initialiser """
        pass

    def reflect(self,fragment):
        tokens = fragment.lower().split()
        for i, token in enumerate(tokens):
            if token in self.reflections:
                tokens[i] = self.reflections[token]
        return ' '.join(tokens)


    def analyze(self,statement):
        for pattern, responses in self.psychobabble:
            match = re.match(pattern, statement.rstrip(".!"))
            if match:
                response = random.choice(responses)
                return response.format(*[self.reflect(g) for g in match.groups()])




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###############################################################################
# capa de abstracción para manejar el html generado por la placa ESP17f weemos
# Copyright © 2020 Valentin Basel <valentinbasel@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################


class PARSER(object):

    """Docstring for PARSER. """

    def __init__(self):
        """TODO: to be defined. """
        self._url = ""
        self._dicc_html = {'REL': '', 'R': '', 'G': '', 'B': ''}

    def recibir(self, arg1):
        """TODO: Docstring for recibir.

        :arg1: TODO
        :returns: TODO

        """
        
    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###############################################################################
#
# base para el sensor dth11 y humedad
# Copyright © 2019 Valentín Basel <valentinbasel@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################


class TEMPERATURA(object):

    """Docstring for HUMEDAD. """
    def __init__(self,  base):
        """TODO: to be defined1. """
        self.__enviar = base

    def leer(self):
        """TODO: Docstring for leer.
        :returns: TODO

        """
        valor = self.__enviar("t")
        try:
            valor = float(valor)
        except Exception as e:
            #print(e)
            valor = "None"
        return valor



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
#
# wrapper para integrar el puerto serie (/dev/ttyACM) mediante usb CDC
# Copyright © 2019 Valentín Basel <valentinbasel@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

import serial
import serial.tools.list_ports


class CDC(object):

    """ docstring clase CDC

    interface de conexión serial USB_CDC para el proyecto de domotica
    DOMOTICARO.
    Implementa una capa de abstracción que permite conectar puerto CDC al
    hardware ICARO o ARDUINO,
    es una API para poder interactuar con el firmware DOMOTICARO_V1.
    su funcion es la de preparar el puerto /devttyUSB o ttyACM para enviar
    y recibir datos.
    una ves inicializada la clase, hay que tener en cuenta lo siguiente:
    el metodo self.puerto es donde se carga el valor del dispositivo serie.
    por defecto es /dev/ttyACM0. cambiarla antes de usar la funcion iniciar()
    """
    _PUERTO = '/dev/ttyACM0'  # valor inicial por defecto
    _BAUDIOS = 9600
    _BYTESIZE = 8
    _PARITY = 'N'
    _STOPBIT = 1
    _TIMEOUT = None
    _XONXOFF = True
    _RTSCTS = True
    _DSRDTR = True
    _RS232 = serial.Serial()

    def __init__(self):
        """TODO: to be defined1. """
        pass

    def puerto(self, arg1):
        """TODO: Docstring for puerto.

        :arg1: String con la dirección del puerto /dev/ttyACM para CDC
        :returns: True / False

        """
        self._PUERTO = arg1

    def iniciar(self):
        """TODO: Docstring for iniciar.

        :arg1: TODO
        :returns: TODO

        """
        self._RS232.port = self._PUERTO
        self._RS232.baudrate = self._BAUDIOS
        self._RS232.bytesize = self._BYTESIZE
        self._RS232.parity = self._PARITY
        self._RS232.stopbit = self._STOPBIT
        self._RS232.timeout = self._TIMEOUT
        self._RS232.xonxoff = self._XONXOFF
        self._RS232.rtscts = self._RTSCTS
        self._RS232.dsrdtr = self._DSRDTR
        self._RS232.exclusive = True
        try:
            self._RS232.open()
            print("iniciar la placa en el puerto :", self._PUERTO)
            return True
        except Exception as e:
            raise e
            return False

    def cerrar(self):
        """TODO: Docstring for cerrar.

        :arg1: TODO
        :returns: TODO

        """
        try:
            self._RS232.close()
            return True
        except:
            return False


    def enviar(self, cadena_caracter):
        """TODO: Docstring for enviar.

        :arg1: TODO
        :returns: TODO

        """
        buff=''
        buff2=""
        if self._RS232.isOpen():
            try:
                for caracter in cadena_caracter:
                    self._RS232.write(caracter.encode("ascii"))
                    #time.sleep(0.01)

                
                while buff != b'\n':
                    buff = self._RS232.read()#self._RS232.inWaiting())
                    buff2 = buff2 + buff.decode("ascii") 
                    #print (buff2)
                #self._RS232.flush()
                return buff2
            except Exception as e:
                print(e)
                print("no se pudo completar el envio de datos")
                return False 
        else:
            print("el puerto: ",self._PUERTO, " esta cerrado")
            return False




#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
#
# interface de abstracción para el hardware basado en micro controladores 
# 18F4550 con bootloader pinguino V4.0
#
# Copyright © 2019 Valentín Basel <valentinbasel@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################
from .base import BASE_HARDWARE 
from .relay import RELE 
#from .analogico import ANALOGICO
#from .lcd import LCD 
class ARDUINO(BASE_HARDWARE):

    def __init__(self,puerto):
        print("inicio ARDUINO")
        BASE_HARDWARE.__init__(self,puerto)
        self.rele1 = RELE(1,self._enviar)
        self.rele2 = RELE(2,self._enviar)
        self.rele3 = RELE(3,self._enviar)
        self.rele4 = RELE(4,self._enviar)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###############################################################################
#
# objeto Rele
# Copyright © 2019 Valentín Basel <valentinbasel@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################


class RELE(object):

    """Docstring for RELE.
        La clase RELE implementa los metodos y parametros necesarios para poder
        controlar un relay mecanico.

        A causa de las limitaciones mecanicas de un relay, no es posible usar
        PWM con la señal.

    """

    def __init__(self, puerto, base):
        """
        puerto = el valor del relay (0,1,2,3) que leera la placa.
        base = clase BASE para poder implementar el envio de datos a la placa.
        """
        self._puerto = puerto
        self.__enviar = base

    def on(self):
        """TODO: Docstring for on.
        Prende el Relay
        :returns: 1

        """
        cadena = "r"+str(self._puerto)+"1"
        self.__enviar(cadena)
        return 1

    def off(self):
        """TODO: Docstring for off.
        apaga el Relay
        :returns: 0

        """
        cadena = "r"+str(self._puerto)+"0"
        self.__enviar(cadena)
        return 0

    def estado(self):
        """TODO: Docstring for estado.
        0 = apagado
        1 = encendido
        :returns: 0 - 1 en función del estado descripto por el rele en la placa

        """
        cadena = "r"+str(self._puerto)+"e"
        r = self.__enviar(cadena)
        if r[0] == '0' or r[0] == '1':
            return int(r)
        else:
            return False

    def conmutar(self):
        """TODO: Docstring for conmutar.
        conmuta el estado del rele. Si el estado es 1, se conmutara a 0 y
        viceversa.
        0 = apagado
        1 = encendido
        :returns: 0 - 1 en función del estado descripto por el rele en la placa
        """
        estado = self.estado()
        if estado == 0:
            return self.on()
        if estado == 1:
            return self.off()
        return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


###############################################################################
# utilerias para chatbot
# Copyright © 2020 Valentin Basel <valentinbasel@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################

import re
import random


class PARES(object):
    def __init__(self, pairs):
        """
        :type pairs: list of tuple
        :param pairs: The patterns and responses
        :type reflections: dict
        :param reflections: A mapping between first and second expressions
        :rtype: None
        """

        reflexiones = {
                     "yo soy": " tu eres",
                     "Yo era": " tú eras",
                     "Yo": "tú",
                     "soy": "eres",
                     "mio": "tuyo",
                     "eras": "yo era",
                     "tu": "lo haré",
                     "tuyo": "mío",
                     "usted": "yo",
                     "yo": "tú",
                     "quiero": "queres",
                     "estoy": "estas",
                     "me": "te"
                        }
        self._pairs = [(re.compile(x, re.IGNORECASE), y) for (x, y) in pairs]
        self._reflections = reflexiones
        self._regex = self._compile_reflections()

    def _compile_reflections(self):
        sorted_refl = sorted(self._reflections.keys(), key=len, reverse=True)
        return re.compile(
            r"\b({0})\b".format("|".join(map(re.escape, sorted_refl))),
            re.IGNORECASE
        )

    def _Comodin(self, response, match):
        pos = response.find("%")
        while pos >= 0:
            num = int(response[pos + 1: pos + 2])
            response = (
                response[:pos]
                + self._substitute(match.group(num))
                + response[pos + 2:]
            )
            pos = response.find("%")
        return response

    def _substitute(self, str):
        """
        Substitute words in the string, according to the specified reflections,
        e.g. "I'm" -> "you are"

        :type str: str
        :param str: The string to be mapped
        :rtype: str
        """

        return self._regex.sub(
            lambda mo: self._reflections[mo.string[mo.start(): mo.end()]],
            str.lower()
        )

    def Respuesta(self, str):
        """
        Generate a response to the user input.

        :type str: str
        :param str: The string to be mapped
        :rtype: str
        """

        # check each pattern
        for (pattern, response) in self._pairs:
            match = pattern.match(str)

            # did the pattern match?
            if match:
                resp = random.choice(response)  # pick a random response
                resp = self._Comodin(resp, match)  # process wildcards

                # fix munged punctuation at the end
                if resp[-2:] == "?.":
                    resp = resp[:-2] + "."
                if resp[-2:] == "??":
                    resp = resp[:-2] + "?"
                return resp



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###############################################################################
# capa de abstracción para parsear HTML
# Copyright © 2020 Valentin Basel <valentinbasel@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################
import requests
from lxml import etree


class HTML(object):

    """Docstring for HTML. """

    def __init__(self):
        """TODO: to be defined. """
        self._url = ""
        self._dicc_html = {'REL': '', 'R': '', 'G': '', 'B': ''}

    def puerto(self, arg):
        """
        """
        self._url = arg
        print("este es el puerto seleccionado: ", arg)

    def iniciar(self):
        """TODO: Docstring for iniciar.

        :arg1: TODO
        :returns: TODO

        """
        pass

    def cerrar(self):
        """TODO: Docstring for cerrar.

        :arg1: TODO
        :returns: TODO

        """
        pass

    def enviar(self, arg1):
        """TODO: Docstring for enviar.

        :arg1: TODO
        :returns: TODO

        """
        if arg1 == "t":
            r = requests.get(self._url, params=self._dicc_html)
            dicc = self.parseador_tabla(r.text)
            return dicc["temperatura"]
        if arg1 == "h":
            r = requests.get(self._url, params=self._dicc_html)
            dicc = self.parseador_tabla(r.text)
            return dicc["humedad"]
        if arg1 == "a0":
            r = requests.get(self._url, params=self._dicc_html)
            dicc = self.parseador_tabla(r.text)
            return dicc["Analogico"]
        if arg1[0] == "r":
            if arg1[2] == "1":
                self._dicc_html["REL"] = "ON"+arg1[1]
                r = requests.get(self._url, params=self._dicc_html)
                dicc = self.parseador_tabla(r.text)
                val = "RELE_"+arg1[1]
                return dicc[val]
            if arg1[2] == "0":
                self._dicc_html["REL"] = "OFF"+arg1[1]
                r = requests.get(self._url, params=self._dicc_html)
                dicc = self.parseador_tabla(r.text)
                val = "RELE_"+arg1[1]
                return dicc[val]
            if arg1[2] == "e":
                self._dicc_html["REL"] = ""
                r = requests.get(self._url, params=self._dicc_html)
                dicc = self.parseador_tabla(r.text)
                val = "RELE_" + arg1[1]
                return dicc[val]
        self._dicc_html["REL"] = ""

    def parseador_tabla(self, arg):
        """TODO: Docstring for parseador_tabla.

        :arg: TODO
        :returns: TODO

        """
        table = etree.HTML(arg).find("body/table")
        rows = iter(table)
        valores = []
        dicc_estado = {}
        for row in rows:
            values = [col.text for col in row]
            valores.append(values)
        for dicc in range(1, len(valores)):
            key, valor = valores[dicc]
            if valor == "ON":
                valor = "1"
            if valor == "OFF":
                valor = "0"
            dicc_estado[key] = valor
        return dicc_estado



#!/usr/bin/env python
# -*- coding: utf-8 -*-

###############################################################################
#
# -roboticaro-
# Copyright © 2017 valentinbasel@gmail.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#
###############################################################################


class ROBOT(object):

    """Docstring for RobotIcaro. """

    def __init__(self, placa):
        """TODO: to be defined1. """
        if placa == "icaro_cdc":
            from .hardware.icaro import PIC18F4550
            self.hardware = PIC18F4550("apicaro")
        elif placa == "arduino":
            from .hardware.arduino import ARDUINO
            self.hardware = ARDUINO("arduino")
        elif placa == "weemos":
            from .hardware.weemos import ESP12F
            self.hardware = ESP12F("weemos")
        else:
            self.hardware = None


def iniciar(placa="icaro_cdc"):
    """TODO: Docstring for iniciar.
    :returns: TODO

    """
    placas = ["icaro_cdc", "arduino", "weemos", None]
    if placa in placas:
        robot = ROBOT(placa)
    else:
        print("la variable :", placa, " no es un valor reconocido")
        print("se inicia el hardware con el emulador")
        robot = ROBOT("emulador")
    return robot.hardware



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###############################################################################
#
# wraper de sintesis de voz espeak
# Copyright © 2019 Valentin Basel <valenitnbasel@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

import subprocess
import re

class VOZ(object):

    """Espeak es una clase para poder implementar un wrapper sencillo que
       permita controlar el programa espeak de los sistemas GNU/linux para
       sintesis de voz por computadora (texto a voz).
       tranforma cualquier cadena de texto en audio.
    """

    def __init__(self,_voz="es",_pitch=100,_velocidad=200):

        """TODO: to be defined1.

        :voz: TODO
        :pitch: TODO
        :velocidad: TODO

        """
        self.args=[
                  ["-v",_voz,str],
                  ["-p",_pitch,int],
                  ["-s",_velocidad,int],
                  ]

    def __ejecutar(self, text):
        """
        metodo privado de la clase para ejecutar mediante subprocess
        el comando espeak con los parametros que se envien.

        :text: cadena de caracteres (string) con el comando espeak a ejecutar
        :returns: True si fue correcto, False si hubo algun error

        """
        cmd = self.__contruir_cmd(text)
        return subprocess.check_output(cmd,
                                       stderr=subprocess.PIPE).decode('UTF8')

    def __contruir_cmd(self, texto):
        """Crea la cadena de caracteres que leera subprocess.

        :texto: texto a insertar en el comando
        :returns: lista con los valores  para ejecutar


        """
        comando_final = ["espeak-ng"]
        for dat in self.args:
            if type(dat[1])==dat[2]:
                comando_final.append(dat[0]+str(dat[1]))
            else:
                print("error")
                return 1
        comando_final.append(texto)
        return comando_final

    def decir(self, texto):
        """TODO: Docstring for decir.

        :texto: cadena de caracteres para traducir a audio
        :returns: True/False

        """
        return self.__ejecutar(str(texto))

    def velocidad(self,v):
        """
        Modifica la velocidad de dicción de Espeak.
        Acepta valores enteros de 10 a 300.
        :v: integer con la velocidad de pronunciacion del texto (10 - 300)
        :returns: True - False

        """
        if type(v)==int:
            if v>=10 and v<=300:
                self.args[2][1]=v
                return True
            else:
                print ("error de valores de velocidad")
                return False
        else:
            print("error, tipo no reconocido. tiene que ser un integer")
            return False

    def pitch(self,v):
        """
        Modifica el tono de dicción de espeak.
        Acepta valores enteros de 0 a 99.
        :v: integer con el timbre de proncunciancion  del texto (0 - 99)
        :returns: True - False

        """
        if type(v)==int:
            if v>0 and v<100:
                self.args[1][1]=v
                return True
            else:
                print ("error de valores del pitch")
                return False
        else:
            print("error, tipo no reconocido. tiene que ser un integer")
            return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###############################################################################
#
# clase base para implementar una capa de abstracción sobre el hardware posible
# para DOMOTICARO
# Copyright © 2019 Valentín Basel <valentinbasel@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################


class BASE_HARDWARE(object):
    """Docstring for BASE_HARDWARE. """

    def __init__(self, puerto="cdc"):
        """ Docstring para __init__
        inicia el puerto de comunicación serial e implementa una capa de
        abstracción para seleccionar el tipo de comunicación entre el hardware
        y PYTHON.
        por defecto inicia con el puerto CDC.

        """
        # print ("inicio la placa 18f4550")
        # esto es medio raro, habria que ver como mejorar, sobre todo si
        # queremos habilitar la placa con otro puerto
        if puerto == "apicaro":
            from .serie import CDC
            print("inicio el hardware con puerto CDC")
            self.__comunicacion = CDC()
        elif puerto == "arduino":
            from .serie_arduino import CDC
            self.__comunicacion = CDC()
        elif puerto == "weemos":
            from .html_weemos import HTML
            self.__comunicacion = HTML()
        else:
            print("error de puerto seleccionado")
            print("se usara el emulador")

    def iniciar(self, puerto="/dev/ttyACM0"):
        """TODO: Docstring for iniciar.
        :returns: TODO

        """
        self.__comunicacion.puerto(puerto)
        return self.__comunicacion.iniciar()

    def puerto(self, arg1):
        """TODO: Docstring for puerto.

        :arg1: TODO
        :returns: TODO

        """
        return self.__comunicacion.puerto(arg1)

    def cerrar(self):
        """TODO: Docstring for cerrar.
        :returns: TODO

        """
        return self.__comunicacion.cerrar()

    def _enviar(self, arg1):
        """TODO: Docstring for _enviar.

        :arg1: TODO
        :returns: TODO

        """
        return self.__comunicacion.enviar(arg1)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###############################################################################
# emulador de chatbot para domoticaro
# Copyright © 2020 Valentin Basel <valentinbasel@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################

import os


class EMULADOR(object):

    def __init__(self, arg=None):
        self.mensajes = []
        self._num_act = 0
        self.chat_id = 1
        self.ruta = os.getenv("HOME") + "/espacio_de_trabajo/emulador/"

    def enviar_imagen(self, chat_id, mens):
        """TODO: Docstring for enviar_imagen.
        :returns: TODO

        """
        mens = "img>> " + mens
        self.enviar_mensaje(chat_id, mens)

    def actualizar(self):
        """TODO: Docstring for actualizar.

        :arg1: TODO
        :returns: TODO

        """
        arch = open(self.ruta + "log1.txt", "r")
        lista_temporal = []
        for dato in arch.readlines():
            lista_temporal.append(dato)
        arch.close()
        self.mensajes = lista_temporal

    def hay_mensajes(self):
        self.actualizar()
        num = len(self.mensajes)
        if num > self._num_act:
            self._num_act = num
            return True
        else:
            return False

    def obtener_ultimo_mensaje(self):
        ultima_act = self._num_act - 1
        mens = self.mensajes[ultima_act]
        chat_id = self.chat_id
        return chat_id, mens

    def enviar_mensaje(self, chat_id, mens):
        # cadena = str(chat_id) + ": " + mens + "\n"
        cadena = "\n" + mens
        arch = open(self.ruta + "log2.txt", "a")
        arch.write(cadena)
        arch.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###############################################################################
#
# clase para controlar la pantalla LCD
# Copyright © 2019 Valentín Basel <valentinbasel@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

class LCD(object):

    """Docstring for LCD. 
        LCD permite comunicar al hardware, a traves del protocolo I2C del 
        hardware, datos de python en la pantallita LCD del hardware.

    """
    def __init__(self,base):
        """
        base = clase BASE para poder implementar el envio de datos a la placa. 
        """
        self.__enviar = base

    def escribir(self, arg1):
        """TODO: Docstring for escribir.

        :arg1: Cadena de texto (string) con la información para enviar al lcd 
        :returns: None

        """
        cadena = "l"+str(arg1)+"\n"
        self.__enviar(cadena)

    def borrar(self):
        """TODO: Docstring for borrar.
        Envia el caracter 'L' para borrar la pantalla del lcd  
        :returns: None

        """
        self.__enviar("L")




from setuptools import setup, find_packages

setup(name="DOMOTICARO",
      version="0.1",
      description="Paquete de instalacion de DOMOTICARO",
      author="Valentin Basel",
      author_email='valentinbasel@gmail.com',
      license="GPL3",
      url="http://roboticaro.org",
      packages=find_packages(),
      #install_requires=[i.strip() for i in
      #                  open("requirements.txt").readlines()],
      #packages=['bot','voz','domoticaro','domoticaro.hardware'],
      install_requires=["requests", "lxml", "pyserial","numpy"]
      )
